<?php

namespace App\WebSocket;

use App\Key\K;
use App\Tools\System;
use EasySwoole\Component\TableManager;
use EasySwoole\EasySwoole\ServerManager;
use Swoole\WebSocket\Server;

class WebSocket
{
    public function message(Server $server, $request)
    {
//        print_r($request);
        $data=$request->data;
        $data=json_decode($data,true);
        if (!$data) return;
        switch ($data['action']){
            case 'connect':
                //第一次连接，那么把数据发回给客户端
                $cache=System::serverData();
                if ($cache) {
                    ServerManager::getInstance()->getSwooleServer()->push($request->fd,json_encode(['action'=>'info','data'=>$cache]));
                }
                break;
        }
    }

    public function open(Server $server, $fd)
    {

    }

    public function close(Server $server, $fd)
    {
        TableManager::getInstance()->get(K::TABLE_WEB_FD)->del($fd);
    }


    /**
     * @param \Swoole\Http\Request $request
     * @param \Swoole\Http\Response $response
     * @return bool
     */
    public function onHandShake(\Swoole\Http\Request $request, \Swoole\Http\Response $response)
    {
        /** 此处自定义握手规则 返回 false 时中止握手 */
        if (!$this->customHandShake($request, $response)) {
            $response->end();
            return false;
        }

        /** 此处是  RFC规范中的WebSocket握手验证过程 必须执行 否则无法正确握手 */
        if ($this->secWebsocketAccept($request, $response)) {
            $response->end();
            return true;
        }

        $response->end();
        return false;
    }

    /**
     * @param \Swoole\Http\Request $request
     * @param \Swoole\Http\Response $response
     * @return bool
     */
    protected function customHandShake(\Swoole\Http\Request $request, \Swoole\Http\Response $response): bool
    {
        /**
         * 这里可以通过 http request 获取到相应的数据
         * 进行自定义验证后即可
         * (注) 浏览器中 JavaScript 并不支持自定义握手请求头 只能选择别的方式 如get参数
         */
//        $headers = $request->header;
//        $cookie = $request->cookie;
//        print_r($request->server);
        //这个 /client 表示是客户端，/server表示服务端
        if (!in_array($request->server['request_uri'], ['/client', '/server'])) return false;
//        switch ($request->server){
//
//        }
        //这里通过判断来只允许某些客户端来访问

        // if (如果不满足我某些自定义的需求条件，返回false，握手失败) {
        //    return false;
        // }
        return true;
    }

    /**
     * RFC规范中的WebSocket握手验证过程
     * 以下内容必须强制使用
     *
     * @param \Swoole\Http\Request $request
     * @param \Swoole\Http\Response $response
     * @return bool
     */
    protected function secWebsocketAccept(\Swoole\Http\Request $request, \Swoole\Http\Response $response): bool
    {
        // ws rfc 规范中约定的验证过程
        if (!isset($request->header['sec-websocket-key'])) {
            // 需要 Sec-WebSocket-Key 如果没有拒绝握手
//            var_dump('shake fai1 3');
            return false;
        }
        if (0 === preg_match('#^[+/0-9A-Za-z]{21}[AQgw]==$#', $request->header['sec-websocket-key'])
            || 16 !== strlen(base64_decode($request->header['sec-websocket-key']))
        ) {
            //不接受握手
//            var_dump('shake fai1 4');
            return false;
        }

        $key = base64_encode(sha1($request->header['sec-websocket-key'] . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11', true));
        $headers = array(
            'Upgrade' => 'websocket',
            'Connection' => 'Upgrade',
            'Sec-WebSocket-Accept' => $key,
            'Sec-WebSocket-Version' => '13',
            'KeepAlive' => 'off',
        );

        if (isset($request->header['sec-websocket-protocol'])) {
            $headers['Sec-WebSocket-Protocol'] = $request->header['sec-websocket-protocol'];
        }
        // 发送验证后的header
        foreach ($headers as $key => $val) {
            $response->header($key, $val);
        }
        // 接受握手 还需要101状态码以切换状态
        $response->status(101);
        //保存这些fd
        $table=TableManager::getInstance()->get(K::TABLE_WEB_FD);
        switch ($request->server['request_uri']) {
            case '/client':
                $table->set($request->fd,['data'=>json_encode(['type'=>'client'])]);
                break;
            case '/server':
                //$table->set($request->fd,['data'=>json_encode(['type'=>'server'])]);
                break;
        }
        return true;
    }
}